/*
 * Copyright (C) 2006 Kouji Sugisawa. All rights reserved.
 */

package jp.sourceforge.livez.pages;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.ext.DefaultHandler2;
import org.xml.sax.helpers.AttributesImpl;

import jp.sourceforge.livez.lang.StringEscapeUtils;

/**
 * @author V _
 */
abstract class AbstractPageHandler extends DefaultHandler2 {

	protected StringBuilder page = new StringBuilder(1024 * 2);

	public final String getPage() {
		return page.toString();
	}

	private boolean seenBody = false;
	private boolean seenPlainText = false;
	private boolean seenXmp = false;

	/*
	 * @see org.xml.sax.ContentHandler#startDocument()
	 */
	@Override
	public void startDocument() throws SAXException {
		seenBody = false;
	}

	/*
	 * @see org.xml.sax.ContentHandler#endDocument()
	 */
	@Override
	public void endDocument() throws SAXException {
		seenBody = false;
	}

	/*
	 * @see org.xml.sax.ContentHandler#startElement(java.lang.String, java.lang.String, java.lang.String, org.xml.sax.Attributes)
	 */
	@Override
	public void startElement(
		final String uri,
		final String localName,
		final String qName,
		final Attributes attributes)
		throws SAXException {

		// body vf̒̂ݏ܂B
		if (qName.equalsIgnoreCase("body") && !seenBody)
			seenBody = true;
		else if (!seenBody)
			return;

		// body vf̊eFw葮⊮܂B
		if ("body".equalsIgnoreCase(qName)) {
			AttributesImpl attrs = new AttributesImpl(attributes);

			if (attrs.getValue("text") == null)
				attrs.addAttribute("", "", "text", "CDATA", "black");
			if (attrs.getValue("link") == null)
				attrs.addAttribute("", "", "link", "CDATA", "blue");
			if (attrs.getValue("alink") == null)
				attrs.addAttribute("", "", "alink", "CDATA", "red");
			if (attrs.getValue("vlink") == null)
				attrs.addAttribute("", "", "vlink", "CDATA", "purple");
			if (attrs.getValue("bgcolor") == null)
				attrs.addAttribute("", "", "bgcolor", "CDATA", "white");

			doStart(uri, localName, qName, attrs);
		} else if ("plaintext".equalsIgnoreCase(qName)) {
			seenPlainText = true;
			doStart(uri, localName, qName, attributes);
		} else if ("xmp".equalsIgnoreCase(qName)) {
			seenXmp = true;
			doStart(uri, localName, qName, attributes);
		} else
			doStart(uri, localName, qName, attributes);
	}

	/**
	 * Jnvf܂B
	 * 
	 * @param uri
	 * @param localName
	 * @param qName
	 * @param attributes
	 * @throws SAXException
	 */
	protected void doStart(
		final String uri,
		final String localName,
		final String qName,
		final Attributes attributes)
		throws SAXException {

		page.append('<');
		page.append(qName);

		for (int i = 0; i < attributes.getLength(); i++) {
			page.append(' ');
			page.append(attributes.getQName(i));
			page.append('=');
			page.append('"');
			page.append(attributes.getValue(i));	// FIXME - Xy[X(")A(')GXP[v
			page.append('"');
		}

		page.append('>');
	}

	/*
	 * @see org.xml.sax.ContentHandler#endElement(java.lang.String, java.lang.String, java.lang.String)
	 */
	@Override
	public final void endElement(
		final String uri,
		final String localName,
		final String qName)
		throws SAXException {

		// body vf̒̂ݏ܂B
		if (!seenBody)
			return;

		// Ivf܂B
		doEnd(uri, localName, qName);

		if ("body".equalsIgnoreCase(qName)) {
			seenBody = false;
			seenPlainText = false;
		} else if ("xmp".equalsIgnoreCase(qName))
			seenXmp = false;
	}

	/**
	 * Ivf܂B
	 * 
	 * @param uri
	 * @param localName
	 * @param qName
	 * @throws SAXException
	 */
	protected void doEnd(
		final String uri,
		final String localName,
		final String qName)
		throws SAXException {

		// PvfłȂꍇ͗vf܂B
		if (!isSimpleElement(qName)) {
			page.append('<');
			page.append('/');
			page.append(qName);
			page.append('>');
		}
	}

	/**
	 * w肳ꂽvfPvfł邩ǂԂ܂B
	 * 
	 * @param tagname vf
	 * @return Pvfł邩ǂ
	 */
	protected static final boolean isSimpleElement(final String tagname) {
		return "meta".equalsIgnoreCase(tagname)
			|| "base".equalsIgnoreCase(tagname)
			|| "br".equalsIgnoreCase(tagname)
			|| "hr".equalsIgnoreCase(tagname)
			|| "img".equalsIgnoreCase(tagname)
			|| "input".equalsIgnoreCase(tagname)
			|| "embed".equalsIgnoreCase(tagname)
			|| "isindex".equalsIgnoreCase(tagname)
			|| "nextid".equalsIgnoreCase(tagname)
			|| "spacer".equalsIgnoreCase(tagname)
			|| "wbr".equalsIgnoreCase(tagname)
			|| "plaintext".equalsIgnoreCase(tagname);
	}

	/*
	 * @see org.xml.sax.ContentHandler#characters(char[], int, int)
	 */
	@Override
	public final void characters(
		final char[] ch,
		final int start,
		final int length)
		/* throws SAXException */ {

		// body vf̒̂ݏ܂B
		if (!seenBody)
			return;

		final String text = new String(ch, start, length);

		if (seenPlainText || seenXmp)
			page.append(text);
		else
			page.append(StringEscapeUtils.escapeHtml(text));
	}

	/*
	 * @see org.xml.sax.ext.LexicalHandler#comment(char[], int, int)
	 */
	@Override
	public final void comment(final char[] ch, final int start, final int length) /* throws SAXException */ {
		// body vf̒̂ݏ܂B
		if (!seenBody)
			return;

		final String comment = "<!--" + new String(ch, start, length) + "-->";
		page.append(comment);
	}

}
