/*
 * Copyright (C) 2005-2006 Kouji Sugisawa. All rights reserved.
 */

package jp.sourceforge.livez.pages;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.net.URLConnection;

import org.apache.commons.lang.StringUtils;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.ext.DefaultHandler2;

import jp.sourceforge.livez.lang.StringEscapeUtils;
import jp.sourceforge.livez.util.URLUtils;

/**
 * t[Zbg̊et[ɑ΂{܂̓AJ[𒊏onhNXłB
 * 
 * @author V _
 */
final class FrameSetHandler extends DefaultHandler2 {

	private StringBuilder page;

	public String getPage() {
		return page.toString();
	}

	/**
	 * t[̑ URL ׂ̊ URL ێ܂B
	 */
	private URL base = null;
	private HTMLOverview header;

	/**
	 * RXgN^łB
	 * 
	 * @param header y[Wwb_
	 */
	public FrameSetHandler(final URL base, final HTMLOverview header) {
		// base vfw肳Ăꍇ́A URL Ɏgp܂B
		if (header.getBaseURI() != null)
			try {
				this.base = header.getBaseURI().toURL();
			} catch (MalformedURLException e) {}	// 

		if (this.base == null)
			this.base = base;

		this.header = header;
	}

	/*
	 * @see org.xml.sax.ContentHandler#startDocument()
	 */
	@Override
	public void startDocument() /* throws SAXException */ {
		page = new StringBuilder(1024 * 2);
		page.append("<html>");
		page.append(header.render());
		page.append("<body>");
	}

	/*
	 * @see org.xml.sax.ContentHandler#endDocument()
	 */
	@Override
	public void endDocument() /* throws SAXException */ {
		page.append("</body>");
		page.append("</html>");
	}

	/*
	 * @see org.xml.sax.ContentHandler#startElement(java.lang.String, java.lang.String, java.lang.String, org.xml.sax.Attributes)
	 */
	@Override
	public void startElement(
		final String uri,
		final String localName,
		final String qName,
		final Attributes attributes)
		/* throws SAXException */ {

		// name  src 擾܂B
		String name = attributes.getValue("name");
		String src = attributes.getValue("src");

		boolean success = false;

		// QƂׂ\[X URL 擾܂B
		URL url = processPath(src);

		if (url != null)
			try {
				success = process(url, src, name);
			} catch (IOException e) {}	// 

		if (!success)
			processAnchor(src, name);
	}

	/**
	 * w肳ꂽ URI Ɗ URL QƂׂ\[X URL \zĕԂ܂B
	 * URL \złȂꍇ <code>null</code> Ԃ܂B
	 * 
	 * @param src URI
	 * @return QƂׂ\[X URL
	 */
	private URL processPath(final String src) {
		try {
			URI uri = new URI(src);
			// src  URI ̏ꍇ baseURI gp URI \z܂
			if (!uri.isAbsolute())
				uri = base.toURI().resolve(uri);

			return uri.toURL();
		} catch (URISyntaxException e) {
			return null;
		} catch (MalformedURLException e) {
			return null;
		}		
	}

	/**
	 * w肳ꂽ\[XQƂď܂B
	 * 
	 * @param url \[XURL
	 * @param src src 
	 * @param name name 
	 * @return K؂ɏꍇ <code>true</code>BȊȌꍇ <code>false</code>
	 * @throws IOException
	 */
	private boolean process(final URL url, final String src, final String name) throws IOException {
		// \[X֐ڑ܂B
		URLConnection con = URLUtils.getURLConnection(url);
		con.connect();

		final String type = con.getContentType();
		if (type == null)
			return false;

		// 摜̏ꍇ
		if (type.startsWith("image/")) {
			// img vf𐶐܂B
			processImage(src, name);
			return true;
		}

		// RecƃRec̕R[h擾܂B
		byte[] bytes = URLUtils.getByteArray(con);
		String charset = URLUtils.getCharSet(con);

		if (type.startsWith("text/plain")) {
			// pre vf𐶐܂B
			try {
				page.append("<div>");
				page.append("<pre>");
				page.append(StringEscapeUtils.escapeHtml(new String(bytes, charset)));
				page.append("</pre>");
				page.append("</div>");
				return true;
			} catch (IOException e) {}	// 

		// HTML ܂ XHTML ̏ꍇ
		} else if (type.startsWith("text/html") ||
			type.startsWith("text/xhtml") ||
			type.startsWith("application/xhtml+xml")) {

			try {
				// y[Wwb_擾܂B
				HTMLOverview ph = HTMLAnalyzer.analyze(bytes, charset);

				// ̎ނ؂܂B
				if (ph.getType() == null) {
					// ĝȂ̏ꍇ͉Ȃ
					return true;
				} else if (ph.getType() == HTMLType.CONTENTS) {
					// ʏ̏̕ꍇ́A
					// body vf div vf֕ϊĐ؎擾܂B
					page.append(FramePageConverter.convert(bytes, charset));
					return true;
				}
			} catch (IOException e) {
				// 
			} catch (SAXException e) {
				// 
			}
		}

		return false;
	}

	private void processImage(final String src, final String name) {
		page.append("<div>");
		page.append("<img src=\"");
		page.append(src);
		page.append('"');

		if (StringUtils.isNotEmpty(name)) {
			page.append(" alt=\"");
			page.append(name);
			page.append('"');
		}

		page.append('>');
		page.append("</div>");
	}

	private void processAnchor(final String src, final String name) {
		page.append("<div>");
		page.append("<a href=\"");
		page.append(src);
		page.append("\">");
		page.append(StringUtils.isNotEmpty(name) ? name : src);
		page.append("</a>");
		page.append("</div>");
	}

}
