/*
 * Copyright (C) 2005-2006 Kouji Sugisawa. All rights reserved.
 */

package jp.sourceforge.livez.util;

import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.net.URL;
import java.net.URLConnection;

import org.apache.commons.codec.DecoderException;
import org.apache.commons.codec.net.URLCodec;
import org.apache.commons.io.IOUtils;

import jp.sourceforge.livez.lang.CharEncoding;

/**
 * HTML `𕄍/ׂ̃[eBeBNXłB
 * 
 * @author V _
 */
public final class URLUtils {

	/**
	 * CX^XłȂ܂B
	 */
	private URLUtils() {}

	/**
	 * x-www-form-urlencoded 𕜍܂B
	 * VtgJIS̕gpāAu%xyv̌`̘AV[PX\肵܂B
	 * 
	 * @param s Ώۂ̕
	 * @return Vꂽ
	 * @throws IllegalArgumentException
	 */
	public static final String decode(final String s) throws IllegalArgumentException {
		if (s == null)
			return null;

		// org.apache.commons.httpclient.util.URIUtil#decode ł
		URLCodec codec = new URLCodec();
		try {
			return codec.decode(s, CharEncoding.SHIFT_JIS);
		} catch (UnsupportedEncodingException e) {
			throw new RuntimeException(e.getMessage(), e);
		} catch (DecoderException e) {
			throw new IllegalArgumentException(e.getMessage(), e);
		}
	}

	/**
	 *  application/x-www-form-urlencoded `ɕϊ܂B
	 * ̃\bh́AVtgJIS̕𕄍ƂĎgpāAO𔭐邱ƂȂ܂B
	 * 
	 * @param s ϊΏۂ̕
	 * @return ϊ̕
	 */
	public static final String encode(final String s) {
		if (s == null)
			return null;

		// org.apache.commons.httpclient.util.URIUtil#encode ł
		URLCodec codec = new URLCodec();
		try {
			return codec.encode(s, CharEncoding.SHIFT_JIS);
		} catch (UnsupportedEncodingException e) {
			throw new RuntimeException(e.getMessage(), e);
		}
	}

	public static URLConnection getURLConnection(final URL url) throws IOException {
		URLConnection con = url.openConnection();
		con.setConnectTimeout(30 * 1000);
		con.setReadTimeout(30 * 1000);
		con.setDoInput(true);
		con.setDoOutput(false);
		con.setUseCaches(false);
//		con.setRequestProperty("User-Agent", "DoCoMo/1.0/N505i/c20/TB/W20H10");
		return con;
	}

	public static byte[] getByteArray(final URL url) throws IOException {
		return getByteArray(getURLConnection(url));
	}

	public static byte[] getByteArray(final URLConnection con) throws IOException {
		InputStream in = con.getInputStream();
		try {
			return IOUtils.toByteArray(in);
		} finally {
			IOUtils.closeQuietly(in);
		}
	}

	public static String getCharSet(final URLConnection con) {
		final String type = con.getContentType();
		if (type == null)
			return null;

		final int i = type.lastIndexOf("charset=");
		if (i == -1)
			return null;

		final String charset = type.substring(i + "charset=".length());
		if ("Shift_JIS".equalsIgnoreCase(charset) || "SJIS".equalsIgnoreCase(charset))
			return "Windows-31J";

		return charset;
	}

}
